import {
  DEFAULT_NAMESPACE,
  stringifyEntityRef,
} from '@backstage/catalog-model';
import {
  createRouter,
  providers,
  defaultAuthProviderFactories,
  SignInInfo,
  AuthResolverContext,
} from '@backstage/plugin-auth-backend';
import { Router } from 'express';
import { PluginEnvironment } from '../types';

const unknownUserSignInResolver = async (
  { profile }: SignInInfo<unknown>,
  ctx: AuthResolverContext,
) => {
  if (!profile.email) {
    throw new Error('Login failed, user profile does not contain an email');
  }

  //const [_, localPart] = profile.email.split('@');

  const userEntityRef = stringifyEntityRef({
    kind: 'User',
    name: 'kjuulh',
    namespace: DEFAULT_NAMESPACE,
  });

  return ctx.issueToken({
    claims: {
      sub: userEntityRef,
      ent: [userEntityRef],
    },
  });
};

export default async function createPlugin(
  env: PluginEnvironment,
): Promise<Router> {
  return await createRouter({
    logger: env.logger,
    config: env.config,
    database: env.database,
    discovery: env.discovery,
    tokenManager: env.tokenManager,
    providerFactories: {
      ...defaultAuthProviderFactories,
      github: providers.github.create({
        signIn: {
          //resolver(_, ctx) {
          //  const userRef = 'user:default/guest'; // Must be a full entity reference
          //  return ctx.issueToken({
          //    claims: {
          //      sub: userRef, // The user's own identity
          //      ent: [userRef], // A list of identities that the user claims ownership through
          //    },
          //  });
          //},
          resolver: unknownUserSignInResolver,
          //resolver: providers.github.resolvers.usernameMatchingUserEntityName(),
        },
      }),
    },
  });
}
